{}
{ QfelExtendedDialogsPL                                                        }
{ Okienka dialogowe sluce do otwierania i zapisywania plikw graficznych     }
{ (C)1998 by Marcin Qfel Zaleski                                               }
{ Based on ExtDlgs.pas Copyright (C) 1995,98 Inprise Corporation               }
{}
{ QfelExtendedDialogsPL.pas (C)1998 by Marcin Qfel Zaleski                     }
{}
{ Changes made to the original Borland's file:                                 }
{ - new property: PanelBevelInner - changes inner bevel of TPanel component    }
{   within the dialog window                                                   }
{ - new property: PanelBevelOuter - changes outer bevel of TPanel component    }
{   within the dialog window                                                   }
{ - new property: WindowImageMargin - initial margin between TImage and inner  }
{   bevel of TScrollBox within the TPreviewWindow                              }
{ - changed routine: PaintBoxPaint                                             }
{ - changed routine: PreviewClick                                              }
{ - PreviewButton placement improved                                           }
{ - some minor changes                                                         }
{}
{ 1998-11-17 started                                                           }
{ 1998-11-20 finished                                                          }
{ 1999-07-24 PreviewButton placement improved
{}
{ Known bugs:                                                                  }
{ - PreviewButton is wrong positioned when dialog is first time displayed with }
{   large font set                                                             }
{}

unit QfelExtendedDialogsPL;

{$R-}

interface

uses
  Dialogs,Graphics,ExtCtrls,StdCtrls,Buttons,Classes,Controls;

type
{}
{ TQfelOpenPictureDialogPL - variation of Borland's TOpenPictureDialog with    }
{                            changed preview and window preview routines,      }
{                            polish interface,added two bevel properties       }
{}
  TQfelOpenPictureDialogPL = class(TOpenDialog)
    private
      FWindowImageMargin : Integer;
      FPanelBevelInner   : TPanelBevel;
      FPanelBevelOuter   : TPanelBevel;
      FPaintPanel        : TPanel;
      FPicture           : TPicture;
      FPicturePanel      : TPanel;
      FPictureLabel      : TLabel;
      FPreviewButton     : TSpeedButton;
      FPaintBox          : TPaintBox;
      procedure SetFPanelBevelInner(NewValue:TPanelBevel);
      procedure SetFPanelBevelOuter(NewValue:TPanelBevel);
      procedure PaintBoxPaint(Sender:TObject);
      procedure PreviewClick(Sender:TObject);
    protected
      procedure DoClose; override;
      procedure DoSelectionChange; override;
      procedure DoShow; override;
    public
      constructor Create(AOwner:TComponent); override;
      destructor Destroy; override;
      function Execute:Boolean; override;
    published
      property PanelBevelInner:TPanelBevel read FPanelBevelInner write SetFPanelBevelInner;
      property PanelBevelOuter:TPanelBevel read FPanelBevelOuter write SetFPanelBevelOuter;
      property WindowImageMargin:Integer read FWindowImageMargin write FWindowImageMargin;
  end;
{}
{ TQfelSavePictureDialogPL - Borland's TSavePictureDialog replacement          }
{}
  TQfelSavePictureDialogPL = class(TQfelOpenPictureDialogPL)
    public
      function Execute:Boolean; override;
  end;
{}
{ Register - registers all classes in 'Qfel' tab                               }
{}
procedure Register;

implementation

uses
  Windows,SysUtils,Forms,CommDlg,QfelPreviewWindow;

{$R QfelExtendedDialogsPL.res}

{==============================================================================}
{ TQfelOpenPictureDialogPL methods                                             }
{==============================================================================}

{}
{ Create - constructs all the needed components,position them and initialize   }
{}
constructor TQfelOpenPictureDialogPL.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPanelBevelInner := bvNone;
  FPanelBevelOuter := bvLowered;
  Filter := GraphicFilter(TGraphic);
  FPicture := TPicture.Create;
  FPicturePanel := TPanel.Create(Self);
  with FPicturePanel do
    begin
      Name := 'PicturePanel';
      Caption := '';
      BevelOuter := bvNone;
      BorderWidth := 6;
      TabOrder := 1;
      FPictureLabel := TLabel.Create(Self);
      with FPictureLabel do
        begin
          Name := 'PictureLabel';
          Caption := '';
          SetBounds(6,6,157,23);
          Align := alTop;
          AutoSize := False;
          Parent := FPicturePanel;
        end;
      FPaintPanel := TPanel.Create(Self);
      with FPaintPanel do
        begin
          Name := 'PaintPanel';
          Caption := '';
          SetBounds(6,29,157,145);
          Align := alClient;
          BevelInner := FPanelBevelInner;;
          BevelOuter := FPanelBevelOuter;
          TabOrder := 0;
          Parent := FPicturePanel;
          FPaintBox := TPaintBox.Create(Self);
          with FPaintBox do
            begin
              Name := 'PaintBox';
              SetBounds(0,0,153,141);
              Align := alClient;
              OnDblClick := PreviewClick;
              OnPaint := PaintBoxPaint;
              Parent := FPaintPanel;
            end;
        end;
      FPreviewButton := TSpeedButton.Create(Self);
      with FPreviewButton do
        begin
          Name := 'PreviewButton';
          SetBounds(0,1,23,22);
          Parent := FPicturePanel;
          Enabled := False;
          Glyph.LoadFromResourceName(HInstance,'BMP_QFEL_PREVIEW');
          Hint := 'Podgld w oknie';
          ParentShowHint := False;
          ShowHint := True;
          OnClick := PreviewClick;
        end;
    end;
end;
{}
{ Destroy - free all the fields objects                                        }
{}
destructor TQfelOpenPictureDialogPL.Destroy;
begin
  FPaintBox.Free;
  FPaintPanel.Free;
  FPreviewButton.Free;
  FPictureLabel.Free;
  FPicturePanel.Free;
  FPicture.Free;
  inherited Destroy;
end;
{}
{ SetFPanelBevelOuter - sets PaintPanel outer bevel                            }
{}
procedure TQfelOpenPictureDialogPL.SetFPanelBevelOuter(NewValue:TPanelBevel);
begin
  if NewValue<>FPanelBevelOuter then
    begin
      FPanelBevelOuter := NewValue;
      FPaintPanel.BevelOuter := NewValue;
    end;
end;
{}
{ SetFPanelBevelInner - sets PaintPanel inner bevel                            }
{}
procedure TQfelOpenPictureDialogPL.SetFPanelBevelInner(NewValue:TPanelBevel);
begin
  if NewValue<>FPanelBevelInner then
    begin
      FPanelBevelInner := NewValue;
      FPaintPanel.BevelInner := NewValue;
    end;
end;
{}
{ DoSelectionChange - responds to selection changes made by user               }
{}
procedure TQfelOpenPictureDialogPL.DoSelectionChange;
var
  FullName     : string;
  ValidPicture : Boolean;

  function ValidFile(const FileName: string): Boolean;
  begin
    Result := GetFileAttributes(PChar(FileName)) <> $FFFFFFFF;
  end;

begin
  Screen.Cursor := crHourGlass;
  FullName := FileName;
  ValidPicture := FileExists(FullName) and ValidFile(FullName);
  if ValidPicture then
    try
      FPicture.LoadFromFile(FullName);
      FPictureLabel.Caption := Format('Rozmiar: %d%d',[FPicture.Width,FPicture.Height]);
      FPreviewButton.Enabled := True;
    except
      ValidPicture := False;
    end;
  if not ValidPicture then
    begin
      FPictureLabel.Caption := '';
      FPreviewButton.Enabled := False;
      FPicture.Assign(nil);
    end;
  FPaintBox.Invalidate;
  inherited DoSelectionChange;
  Screen.Cursor := crDefault;
end;
{}
{ DoClose - closes dialog                                                      }
{}
procedure TQfelOpenPictureDialogPL.DoClose;
begin
  inherited DoClose;
  { Hide any hint windows left behind }
  Application.HideHint;
end;
{}
{ DoShow - shows dialog                                                        }
{}
procedure TQfelOpenPictureDialogPL.DoShow;
var
  PreviewRect : TRect;
  StaticRect  : TRect;
begin
  { Set preview area to entire dialog }
  GetClientRect(Handle,PreviewRect);
  StaticRect := GetStaticRect;
  { Move preview area to right of static area }
  PreviewRect.Left := StaticRect.Left + (StaticRect.Right - StaticRect.Left);
  Inc(PreviewRect.Top,4);
  FPicturePanel.BoundsRect := PreviewRect;
  FPreviewButton.Left := FPaintPanel.BoundsRect.Right - FPreviewButton.Width - 2;  
  FPicture.Assign(nil);
  FPicturePanel.ParentWindow := Handle;
  inherited DoShow;
end;
{}
{ Execute - prepares resources and shows dialog modally                        }
{}
function TQfelOpenPictureDialogPL.Execute;
begin
  if NewStyleControls and not (ofOldStyleDialog in Options) then
    Template := 'DIALOG_QFEL_OPENSAVEPICT'
  else
    Template := nil;
  Result := inherited Execute;
end;
{}
{ PaintBoxPaint - paints scaled image on a dialog box, completly rebuilded     }
{                 routine                                                      }
{}
procedure TQfelOpenPictureDialogPL.PaintBoxPaint(Sender: TObject);
const
  IDS_BRAK = '<podgld niedostpny>';
var
  DrawRect      : TRect;
  RectAspect    : Double;
  ImageAspect   : Double;
  OldRectWidth  : Integer;
  OldRectHeight : Integer;
  NewRectWidth  : Integer;
  NewRectHeight : Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    with TPaintBox(Sender) do
      begin
        DrawRect := ClientRect;
        if FPicture.Width > 0 then
          begin
            InflateRect(DrawRect,-2,-2);
            OldRectWidth := DrawRect.Right - DrawRect.Left;
            OldRectHeight := DrawRect.Bottom - DrawRect.Top;
            if ((FPicture.Width<=OldRectWidth)and(FPicture.Height<=OldRectHeight)) then
              with DrawRect do
                Canvas.Draw(Left+(Right-Left-FPicture.Width) div 2,Top+(Bottom-Top-FPicture.Height) div 2,FPicture.Graphic)
            else
              begin
                RectAspect := OldRectWidth / OldRectHeight;
                ImageAspect := FPicture.Width / FPicture.Height;
                if RectAspect>ImageAspect then
                  begin
                    NewRectWidth := MulDiv(FPicture.Width,OldRectHeight,FPicture.Height);
                    InflateRect(DrawRect,-((OldRectWidth-NewRectWidth) div 2),0);
                  end
                else
                  begin
                    NewRectHeight := MulDiv(FPicture.Height,OldRectWidth,FPicture.Width);
                    InflateRect(DrawRect,0,-((OldRectHeight-NewRectHeight) div 2));
                  end;
                Canvas.StretchDraw(DrawRect,FPicture.Graphic);
              end;
          end
        else
          with DrawRect,Canvas do
            TextOut(Left+(Right-Left-TextWidth(IDS_BRAK)) div 2,Top+(Bottom-Top-TextHeight(IDS_BRAK)) div 2,IDS_BRAK);
      end;
  finally
    Screen.Cursor := crDefault;
  end;
end;
{}
{ PreviewClick - full size preview in window                                   }
{}
procedure TQfelOpenPictureDialogPL.PreviewClick(Sender: TObject);
begin
  if not FPreviewButton.Enabled then
    System.Exit;
  Screen.Cursor := crHourGlass;
  with TQfelPrevWindow.Create(Self) do
    try
      ImageMargin := WindowImageMargin;
      Caption := FileName;
      Image.Picture.Assign(FPicture);
      Screen.Cursor := crDefault;
      ShowModal;
    finally
      Free;
    end;
end;

{==============================================================================}
{ TQfelOpenPictureDialogPL methods                                             }
{==============================================================================}

{}
{ Execute - prepares resources and shows dialog modally                        }
{}
function TQfelSavePictureDialogPL.Execute: Boolean;
begin
  if NewStyleControls and not (ofOldStyleDialog in Options) then
    Template := 'DIALOG_QFEL_OPENSAVEPICT'
  else
    Template := nil;
  Result := DoExecute(@GetSaveFileName);
end;

{}
{ Register - registers all components                                          }
{}
procedure Register;
begin
  RegisterComponents('Qfel',[TQfelOpenPictureDialogPL,TQfelSavePictureDialogPL]);
end;

end.

